(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    (global = global || self, global.LeafletSvelte = factory());
}(this, (function () { 'use strict';

    function noop() { }
    function run(fn) {
        return fn();
    }
    function blank_object() {
        return Object.create(null);
    }
    function run_all(fns) {
        fns.forEach(run);
    }
    function is_function(thing) {
        return typeof thing === 'function';
    }
    function safe_not_equal(a, b) {
        return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
    }
    function is_empty(obj) {
        return Object.keys(obj).length === 0;
    }
    function subscribe(store, ...callbacks) {
        if (store == null) {
            return noop;
        }
        const unsub = store.subscribe(...callbacks);
        return unsub.unsubscribe ? () => unsub.unsubscribe() : unsub;
    }
    function component_subscribe(component, store, callback) {
        component.$$.on_destroy.push(subscribe(store, callback));
    }

    function append(target, node) {
        target.appendChild(node);
    }
    function insert(target, node, anchor) {
        target.insertBefore(node, anchor || null);
    }
    function detach(node) {
        node.parentNode.removeChild(node);
    }
    function element(name) {
        return document.createElement(name);
    }
    function text(data) {
        return document.createTextNode(data);
    }
    function space() {
        return text(' ');
    }
    function listen(node, event, handler, options) {
        node.addEventListener(event, handler, options);
        return () => node.removeEventListener(event, handler, options);
    }
    function attr(node, attribute, value) {
        if (value == null)
            node.removeAttribute(attribute);
        else if (node.getAttribute(attribute) !== value)
            node.setAttribute(attribute, value);
    }
    function children(element) {
        return Array.from(element.childNodes);
    }
    function custom_event(type, detail) {
        const e = document.createEvent('CustomEvent');
        e.initCustomEvent(type, false, false, detail);
        return e;
    }

    let current_component;
    function set_current_component(component) {
        current_component = component;
    }
    function get_current_component() {
        if (!current_component)
            throw new Error(`Function called outside component initialization`);
        return current_component;
    }
    function onMount(fn) {
        get_current_component().$$.on_mount.push(fn);
    }
    function createEventDispatcher() {
        const component = get_current_component();
        return (type, detail) => {
            const callbacks = component.$$.callbacks[type];
            if (callbacks) {
                // TODO are there situations where events could be dispatched
                // in a server (non-DOM) environment?
                const event = custom_event(type, detail);
                callbacks.slice().forEach(fn => {
                    fn.call(component, event);
                });
            }
        };
    }

    const dirty_components = [];
    const binding_callbacks = [];
    const render_callbacks = [];
    const flush_callbacks = [];
    const resolved_promise = Promise.resolve();
    let update_scheduled = false;
    function schedule_update() {
        if (!update_scheduled) {
            update_scheduled = true;
            resolved_promise.then(flush);
        }
    }
    function add_render_callback(fn) {
        render_callbacks.push(fn);
    }
    let flushing = false;
    const seen_callbacks = new Set();
    function flush() {
        if (flushing)
            return;
        flushing = true;
        do {
            // first, call beforeUpdate functions
            // and update components
            for (let i = 0; i < dirty_components.length; i += 1) {
                const component = dirty_components[i];
                set_current_component(component);
                update(component.$$);
            }
            dirty_components.length = 0;
            while (binding_callbacks.length)
                binding_callbacks.pop()();
            // then, once components are updated, call
            // afterUpdate functions. This may cause
            // subsequent updates...
            for (let i = 0; i < render_callbacks.length; i += 1) {
                const callback = render_callbacks[i];
                if (!seen_callbacks.has(callback)) {
                    // ...so guard against infinite loops
                    seen_callbacks.add(callback);
                    callback();
                }
            }
            render_callbacks.length = 0;
        } while (dirty_components.length);
        while (flush_callbacks.length) {
            flush_callbacks.pop()();
        }
        update_scheduled = false;
        flushing = false;
        seen_callbacks.clear();
    }
    function update($$) {
        if ($$.fragment !== null) {
            $$.update();
            run_all($$.before_update);
            const dirty = $$.dirty;
            $$.dirty = [-1];
            $$.fragment && $$.fragment.p($$.ctx, dirty);
            $$.after_update.forEach(add_render_callback);
        }
    }
    const outroing = new Set();
    let outros;
    function transition_in(block, local) {
        if (block && block.i) {
            outroing.delete(block);
            block.i(local);
        }
    }
    function transition_out(block, local, detach, callback) {
        if (block && block.o) {
            if (outroing.has(block))
                return;
            outroing.add(block);
            outros.c.push(() => {
                outroing.delete(block);
                if (callback) {
                    if (detach)
                        block.d(1);
                    callback();
                }
            });
            block.o(local);
        }
    }

    const globals = (typeof window !== 'undefined'
        ? window
        : typeof globalThis !== 'undefined'
            ? globalThis
            : global);
    function create_component(block) {
        block && block.c();
    }
    function mount_component(component, target, anchor) {
        const { fragment, on_mount, on_destroy, after_update } = component.$$;
        fragment && fragment.m(target, anchor);
        // onMount happens before the initial afterUpdate
        add_render_callback(() => {
            const new_on_destroy = on_mount.map(run).filter(is_function);
            if (on_destroy) {
                on_destroy.push(...new_on_destroy);
            }
            else {
                // Edge case - component was destroyed immediately,
                // most likely as a result of a binding initialising
                run_all(new_on_destroy);
            }
            component.$$.on_mount = [];
        });
        after_update.forEach(add_render_callback);
    }
    function destroy_component(component, detaching) {
        const $$ = component.$$;
        if ($$.fragment !== null) {
            run_all($$.on_destroy);
            $$.fragment && $$.fragment.d(detaching);
            // TODO null out other refs, including component.$$ (but need to
            // preserve final state?)
            $$.on_destroy = $$.fragment = null;
            $$.ctx = [];
        }
    }
    function make_dirty(component, i) {
        if (component.$$.dirty[0] === -1) {
            dirty_components.push(component);
            schedule_update();
            component.$$.dirty.fill(0);
        }
        component.$$.dirty[(i / 31) | 0] |= (1 << (i % 31));
    }
    function init(component, options, instance, create_fragment, not_equal, props, dirty = [-1]) {
        const parent_component = current_component;
        set_current_component(component);
        const prop_values = options.props || {};
        const $$ = component.$$ = {
            fragment: null,
            ctx: null,
            // state
            props,
            update: noop,
            not_equal,
            bound: blank_object(),
            // lifecycle
            on_mount: [],
            on_destroy: [],
            before_update: [],
            after_update: [],
            context: new Map(parent_component ? parent_component.$$.context : []),
            // everything else
            callbacks: blank_object(),
            dirty,
            skip_bound: false
        };
        let ready = false;
        $$.ctx = instance
            ? instance(component, prop_values, (i, ret, ...rest) => {
                const value = rest.length ? rest[0] : ret;
                if ($$.ctx && not_equal($$.ctx[i], $$.ctx[i] = value)) {
                    if (!$$.skip_bound && $$.bound[i])
                        $$.bound[i](value);
                    if (ready)
                        make_dirty(component, i);
                }
                return ret;
            })
            : [];
        $$.update();
        ready = true;
        run_all($$.before_update);
        // `false` as a special case of no DOM component
        $$.fragment = create_fragment ? create_fragment($$.ctx) : false;
        if (options.target) {
            if (options.hydrate) {
                const nodes = children(options.target);
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                $$.fragment && $$.fragment.l(nodes);
                nodes.forEach(detach);
            }
            else {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                $$.fragment && $$.fragment.c();
            }
            if (options.intro)
                transition_in(component.$$.fragment);
            mount_component(component, options.target, options.anchor);
            flush();
        }
        set_current_component(parent_component);
    }
    class SvelteComponent {
        $destroy() {
            destroy_component(this, 1);
            this.$destroy = noop;
        }
        $on(type, callback) {
            const callbacks = (this.$$.callbacks[type] || (this.$$.callbacks[type] = []));
            callbacks.push(callback);
            return () => {
                const index = callbacks.indexOf(callback);
                if (index !== -1)
                    callbacks.splice(index, 1);
            };
        }
        $set($$props) {
            if (this.$$set && !is_empty($$props)) {
                this.$$.skip_bound = true;
                this.$$set($$props);
                this.$$.skip_bound = false;
            }
        }
    }

    function loader (urls, test, callback) {
      let remaining = urls.length;

      function maybeCallback () {
        remaining = --remaining;
        if (remaining < 1) {
          callback();
        }
      }

      if (!test()) {
        urls.forEach(({ type, url, options = { async: true, defer: true }}) => {
          const isScript = type === 'script';
          const tag = document.createElement(isScript ? 'script': 'link');
          if (isScript) {
            tag.src = url;
            tag.async = options.async;
            tag.defer = options.defer;
          } else {
            tag.rel = 'stylesheet';
    		    tag.href = url;
          }
          tag.onload = maybeCallback;
          document.body.appendChild(tag);
        });
      } else {
        callback();
      }
    }

    const subscriber_queue = [];
    /**
     * Create a `Writable` store that allows both updating and reading by subscription.
     * @param {*=}value initial value
     * @param {StartStopNotifier=}start start and stop notifications for subscriptions
     */
    function writable(value, start = noop) {
        let stop;
        const subscribers = [];
        function set(new_value) {
            if (safe_not_equal(value, new_value)) {
                value = new_value;
                if (stop) { // store is ready
                    const run_queue = !subscriber_queue.length;
                    for (let i = 0; i < subscribers.length; i += 1) {
                        const s = subscribers[i];
                        s[1]();
                        subscriber_queue.push(s, value);
                    }
                    if (run_queue) {
                        for (let i = 0; i < subscriber_queue.length; i += 2) {
                            subscriber_queue[i][0](subscriber_queue[i + 1]);
                        }
                        subscriber_queue.length = 0;
                    }
                }
            }
        }
        function update(fn) {
            set(fn(value));
        }
        function subscribe(run, invalidate = noop) {
            const subscriber = [run, invalidate];
            subscribers.push(subscriber);
            if (subscribers.length === 1) {
                stop = start(set) || noop;
            }
            run(value);
            return () => {
                const index = subscribers.indexOf(subscriber);
                if (index !== -1) {
                    subscribers.splice(index, 1);
                }
                if (subscribers.length === 0) {
                    stop();
                    stop = null;
                }
            };
        }
        return { set, update, subscribe };
    }

    const mapsLoaded = writable(false);
    const mapsLoading = writable(false);

    /* src\LoadSdk.svelte generated by Svelte v3.24.1 */

    function instance($$self, $$props, $$invalidate) {
    	let $mapsLoaded;
    	let $mapsLoading;
    	component_subscribe($$self, mapsLoaded, $$value => $$invalidate(2, $mapsLoaded = $$value));
    	component_subscribe($$self, mapsLoading, $$value => $$invalidate(3, $mapsLoading = $$value));
    	const dispatch = createEventDispatcher();
    	let L = {};
    	let map = "";

    	onMount(() => {
    		if ($mapsLoaded) {
    			dispatch("ready");
    		}

    		if (!$mapsLoading) {
    			mapsLoading.set(true);

    			loader(
    				[
    					{
    						type: "style",
    						url: `https://unpkg.com/leaflet@1.7.1/dist/leaflet.css`
    					},
    					{
    						type: "script",
    						url: `https://unpkg.com/leaflet@1.7.1/dist/leaflet.js`
    					}
    				],
    				() => {
    					return false;
    				},
    				() => {
    					L = window.L;
    					map = L.map;
    					mapsLoaded.set(true);
    					return true;
    				}
    			);
    		}
    	});

    	$$self.$$.update = () => {
    		if ($$self.$$.dirty & /*$mapsLoaded*/ 4) {
    			 $mapsLoaded && dispatch("ready");
    		}
    	};

    	return [];
    }

    class LoadSdk extends SvelteComponent {
    	constructor(options) {
    		super();
    		init(this, options, instance, null, safe_not_equal, {});
    	}
    }

    /* src\Component.svelte generated by Svelte v3.24.1 */

    const { window: window_1 } = globals;

    function add_css() {
    	var style = element("style");
    	style.id = "svelte-1xdqv5q-style";
    	style.textContent = ".map.svelte-1xdqv5q{height:inherit;width:inherit}";
    	append(document.head, style);
    }

    function create_fragment(ctx) {
    	let div;
    	let t;
    	let loadsdk;
    	let current;
    	let mounted;
    	let dispose;
    	loadsdk = new LoadSdk({});
    	loadsdk.$on("ready", /*initialise*/ ctx[1]);

    	return {
    		c() {
    			div = element("div");
    			t = space();
    			create_component(loadsdk.$$.fragment);
    			attr(div, "class", "map svelte-1xdqv5q");
    			attr(div, "id", /*mapID*/ ctx[0]);
    		},
    		m(target, anchor) {
    			insert(target, div, anchor);
    			/*div_binding*/ ctx[4](div);
    			insert(target, t, anchor);
    			mount_component(loadsdk, target, anchor);
    			current = true;

    			if (!mounted) {
    				dispose = listen(window_1, "resize", /*resizeMap*/ ctx[2]);
    				mounted = true;
    			}
    		},
    		p(ctx, [dirty]) {
    			if (!current || dirty & /*mapID*/ 1) {
    				attr(div, "id", /*mapID*/ ctx[0]);
    			}
    		},
    		i(local) {
    			if (current) return;
    			transition_in(loadsdk.$$.fragment, local);
    			current = true;
    		},
    		o(local) {
    			transition_out(loadsdk.$$.fragment, local);
    			current = false;
    		},
    		d(detaching) {
    			if (detaching) detach(div);
    			/*div_binding*/ ctx[4](null);
    			if (detaching) detach(t);
    			destroy_component(loadsdk, detaching);
    			mounted = false;
    			dispose();
    		}
    	};
    }

    function instance$1($$self, $$props, $$invalidate) {
    	let L = {};
    	let map = "";
    	const dispatch = createEventDispatcher();
    	let { options } = $$props;

    	let { zoom = 13, maxZoom = 19, minZoom = 1, mapID = "map", attributionControl = true, center = [0, 0], markers, recenter = false, scrollWheelZoom = true, tileLayer = {
    		url: "https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png",
    		attribution: "&copy; <a href=\"https://www.openstreetmap.org/copyright\">OpenStreetMap</a> contributors"
    	}, controls = {
    		zoomControl: true,
    		position: "topleft",
    		scale: false
    	} } = options;

    	function initialise() {
    		setTimeout(
    			() => {
    				L = window["L"];
    				createMap();
    				dispatch("ready");
    			},
    			1
    		);
    	}

    	function createMap() {
    		map = L.map(mapID, {
    			attributionControl,
    			zoomControl: controls.zoomControl,
    			minZoom,
    			maxZoom
    		}).setView(center, zoom);

    		L.tileLayer(tileLayer.url, { attribution: tileLayer.attribution }).addTo(map);

    		if (!scrollWheelZoom) {
    			map.scrollWheelZoom.disable();
    		}

    		let controlElement = L.control;

    		if (!controls.zoomControl) {
    			controlElement().remove();
    		}

    		if (controls.scale) {
    			controlElement.scale({ position: controls.position }).addTo(map);
    		}

    		if (controls.zoomControl && controls.position) {
    			map.removeControl(map.zoomControl);
    			controlElement.zoom({ position: controls.position }).addTo(map);
    		}
    		let icon;
    		let markersArray = [];
    		let bounds;

    		let defaultIcon = {
    			iconUrl: "https://unpkg.com/leaflet@1.7.1/dist/images/marker-icon.png",
    			iconRetinaUrl: "https://unpkg.com/leaflet@1.7.1/dist/images/marker-icon-2x.png",
    			shadowUrl: "https://unpkg.com/leaflet@1.7.1/dist/images/marker-shadow.png",
    			iconSize: [25, 41],
    			iconAnchor: [12, 41],
    			popupAnchor: [1, -34],
    			tooltipAnchor: [16, -28],
    			shadowSize: [41, 41]
    		};

    		if (markers) {
    			markers.map(e => {
    				markersArray.push([e.lat, e.lng]);

    				if (e.icon) {
    					icon = L.icon(e.icon);
    				} else {
    					icon = L.icon(defaultIcon);
    				}

    				let marker = new L.marker([e.lat, e.lng], { icon });

    				if (e.popup) {
    					marker.bindPopup(e.popup.text, { closeOnClick: false, autoClose: false }).addTo(map);

    					if (e.popup.isOpen) {
    						marker.openPopup();
    					}
    				}

    				marker.addTo(map);
    			});

    			if (recenter) {
    				if (markersArray.length == 1) {
    					map.panTo(L.latLng(markersArray[0][0], markersArray[0][1]));
    					map.setZoom(zoom);
    				} else {
    					bounds = new L.LatLngBounds(markersArray);
    					map.fitBounds(bounds);
    				}
    			}
    		}
    	}

    	function resizeMap() {
    		if (map) {
    			map.invalidateSize();
    		}
    	}

    	function div_binding($$value) {
    		binding_callbacks[$$value ? "unshift" : "push"](() => {
    			mapID = $$value;
    			$$invalidate(0, mapID);
    		});
    	}

    	$$self.$$set = $$props => {
    		if ("options" in $$props) $$invalidate(3, options = $$props.options);
    	};

    	return [mapID, initialise, resizeMap, options, div_binding];
    }

    class Component extends SvelteComponent {
    	constructor(options) {
    		super();
    		if (!document.getElementById("svelte-1xdqv5q-style")) add_css();
    		init(this, options, instance$1, create_fragment, safe_not_equal, { options: 3 });
    	}
    }

    return Component;

})));
